/* cloaddsk : converts an disk saved in Oric tape image by Savedisk, to a sedoric dsk image */
/* (c) Simon Guyart 2005, derived from F.Frances' Tap2dsk (c) 2004 */

#include <stdio.h>

typedef unsigned char byte;
typedef byte sector[256];

struct {
	char signature[8];
	int sides;
	int tracks;
	int sect;
	char pad[256-20];
} imageheader = { "ORICDISK", 0, 0, 0 };

sector disk[101][19];  //taille maxi du disque
sector sector_buffer;

char debut[]="MainData";
int first_read=0; /* indicateur 1ere lecteur pour recherche MainData */
int total_sides=999; /*nb de faces,  renseigner par MainData */
int total_tracks=999; /*nb de pistes,  renseigner par MainData */
int total_sectors=999; /*nb de secteurs,  renseigner par MainData */
int current_sect=1; /* piste en cours de lecture/ecriture */
int current_track=1; /* piste en cours de lecture/ecriture */
int current_side=1; /* face en cours de lecture/ecriture */
int header_side=0;
int header_track=0;
int logic_physic=0; /* (inutilis pr le moment) dans MainData: transfert physique ou logique de la disquette */


int main(int argc, char *argv[])
{
	int track_counter=0;
	byte header[9];
	char name[17];
	byte file_buffer[48*1024];
	FILE *tape, *dsk;

	if (argc!=3) {
		fprintf(stderr,"Usage: cloaddsk <tape_image> <old_disk_image>\n");
		exit(1);
	}
	tape=fopen(argv[1],"rb");
	if (tape==NULL) {
		fprintf(stderr,"Cannot read tape image %s\n",argv[1]);
		exit(1);
	}
	dsk=fopen(argv[2],"wb");
	if (dsk==NULL) {
		fprintf(stderr,"Cannot open %s for writing\n",argv[2]);
		exit(1);
	}

	while (fgetc(tape)!=EOF) {
		int start,end,i;
		while (fgetc(tape)!=0x24)
			;
		for (i=0;i<9;i++) header[i]=fgetc(tape);
		for (i=0;i<17;i++) {
			name[i]=fgetc(tape);
			if (name[i]==0) break;
		}

		start=(header[6]<<8)+header[7];
		end=(header[4]<<8)+header[5];
		for (i=0; i<end+1-start; i++)
			file_buffer[i]=fgetc(tape);
		if (first_read==0) {
			first_read=1;
			if (strcmp(name, debut)!=0) {
				fprintf(stderr,"Problem: can't find MainData, found %s\n", name);
				exit(1);
			}
			total_tracks=file_buffer[1];
			total_sectors=file_buffer[2];
			if (file_buffer[5] < 127)
				total_sides=1;
			else
				total_sides=2;
			if ((total_tracks > 101)||(total_tracks < 21)||(total_sectors > 19)||(total_sectors < 16)) {
				fprintf(stderr,"Problem: %d tracks and %d sectors to read!\n", total_tracks, total_sectors);
				exit(1);
			}

			printf("Cloaddsk v1.0 --- %d side(s) and %d tracks to read\n",total_sides, total_tracks);

			/* remplissage du header*/
			imageheader.sides=total_sides;
			imageheader.tracks=total_tracks;
			imageheader.sect=total_sectors;

		}
		else
		{	/* vrifier No face et piste; 48 = valeur  retirer de l'ASCII*/
			header_side=name[1]-48;
			header_track=((name[3]-48)*10)+name[4]-48;
			if ((current_track-1!=header_track)||(current_side!=header_side)) {
				fprintf(stderr,"Problem: track %d of side %d not found.\n", current_track-1, current_side);
				exit(1);
			}
			/* remplir la piste */
			for (current_sect=1;current_sect<=total_sectors;current_sect++) {
				for (i=0; i<256; i++)
					disk[current_track+(total_tracks*current_side-1)-1][current_sect-1][i]=file_buffer[i+256*(current_sect-1)];
			}
			current_track++;
			track_counter++;
			if (current_track>total_tracks) {
				current_side++;
				current_track=1;
			}
		}
	}
	//printf("total tracks read %d, total sides read %d, track_counter %d\n", total_tracks, total_sides, track_counter);
	if (track_counter!=total_tracks*total_sides) {
		fprintf(stderr,"Data missing, stopped at track %d of side %d.\n", current_track-1, current_side);
		exit(1);
	}
	else {
		printf("Transfer complete, now use OLD2MFM and you can use your disk!\n");
	}
	fwrite(&imageheader,sizeof(imageheader),1,dsk);
	for (current_side=1;current_side<=total_sides;current_side++)
		for (current_track=1;current_track<=total_tracks;current_track++)
			for (current_sect=1;current_sect<=total_sectors;current_sect++)
			fwrite(disk[current_track+(total_tracks*current_side-1)-1][current_sect-1],sizeof(sector),1,dsk);
}
